/*	AMX Mod X script.

	NoSeeVar plugin by Zetex.

	Description:
		NoSeeVar allows admins to hide cvars from the network requests.
		Plugin will not allow you to hide core Half-Life cvars (for example, amxmodx_version,
		sv_cheats or any other cvar that works as part of a system and needs to be 'seen' by
		other components).

	Commands:
		nsv_list <page> - show list of all hidden cvars.

	ChangeLog:
		v1.0 [08.02.2014]
			Initial release.
		v2.0 [10.02.2014]
			Added nsv_list command.
		v3.0 [12.02.2014]
			Added debug output at server console.
			Fixed small potential bugs.
*/

#pragma semicolon 1
#pragma ctrlchar '\'

#include <amxmodx>
#include <amxmisc>

#define PLUGIN_NAME "NoSeeVar"
#define PLUGIN_TAG "[NSV]"
#define VERSION_STR "3.0"
#define VERSION_CVARNAME "nsv_version"

// Constants:
new const nsv_filename[] = "nsv.ini";
new const list_paginate = 10;
new const default_flag = FCVAR_EXTDLL;

// Global functions:
new Array:nsv_cvarslist, nsv_filedir[64];

public plugin_init() {
	register_plugin(PLUGIN_NAME, VERSION_STR, "Zetex");
	register_cvar(VERSION_CVARNAME, VERSION_STR, default_flag);
	register_concmd("nsv_list", "concmd_List", ADMIN_CFG, "<page> - show list of all hidden cvars");

	nsv_cvarslist = ArrayCreate(32);
	get_configsdir(nsv_filedir, charsmax(nsv_filedir));
	format(nsv_filedir, charsmax(nsv_filedir), "%s/%s", nsv_filedir, nsv_filename);
	load_cvars();
}

public plugin_end() {
	ArrayDestroy(nsv_cvarslist);
}

public load_cvars() {
	new buffer[128], nsv_file = fopen(nsv_filedir, "rt");
	ArrayClear(nsv_cvarslist);

	if (!nsv_file) {
		server_print("%s Couldn't read file \"%s\"", PLUGIN_TAG, nsv_filename);
		return PLUGIN_HANDLED;
	}

	while (!feof(nsv_file)) {
		fgets(nsv_file, buffer, charsmax(buffer));
		trim(buffer);

		if (!buffer[0] || buffer[0] == ';' || buffer[0] == '#') {
			continue;
		}

		if (cvar_exists(buffer)) {
			remove_cvar_flags(buffer);
			set_cvar_flags(buffer, default_flag);
			new newflag = get_cvar_flags(buffer);

			if (newflag != default_flag) {
				server_print("%s Cvar \"%s\" isn't changed [default flag: %d]", PLUGIN_TAG, buffer, newflag);
			} else {
				ArrayPushString(nsv_cvarslist, buffer);
			}
		} else {
			server_print("%s Unknown input \"%s\"", PLUGIN_TAG, buffer);
		}
	}

	fclose(nsv_file);
	return PLUGIN_HANDLED;
}

public concmd_List(id, level, cid) {
	if (!cmd_access(id, level, cid, 1)) {
		return PLUGIN_HANDLED;
	}

	new nsv_arraysize = ArraySize(nsv_cvarslist);

	if (!nsv_arraysize) {
		console_print(id, "%s Nothing is protected", PLUGIN_TAG);
		return PLUGIN_HANDLED;
	}

	new cmdname[32], page[4], start, end;
	read_argv(0, cmdname, charsmax(cmdname));
	start = read_argv(1, page, charsmax(page)) ? str_to_num(page) : 1;

	if (--start < 0) {
		start = 0;
	}

	if (start >= nsv_arraysize) {
		start = nsv_arraysize - 1;
	}

	end = start + list_paginate;

	if (end > nsv_arraysize) {
		end = nsv_arraysize;
	}

	for (new i = start; i < end; i++) {
		console_print(id, "%3d. %a", i + 1, ArrayGetStringHandle(nsv_cvarslist, i));
	}

	console_print(id, "Entries %d - %d of %d", start + 1, end, nsv_arraysize);

	if (end < nsv_arraysize) {
		console_print(id, "Use \"%s %d\" for more", cmdname, end + 1);
	} else {
		console_print(id, "Use \"%s 1\" for begin", cmdname);
	}

	return PLUGIN_HANDLED;
}